/*
 * This demo shows how to use the HEL's map-system for parallax scrolling!
 *
 * Map & graphics borrowed from Nessie's MapEd sample. (http://nessie.gbadev.org/)
 *
*/
/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <mygba.h>
// include the HEL library header
#include <hel.h>

#include "gfx/sky.raw.c"
#include "gfx/sky.map.c"
#include "gfx/platformfar.raw.c"
#include "gfx/platformfar.map.c"
#include "gfx/platform.raw.c"
#include "gfx/platform.map.c"
#include "gfx/master.pal.c"

/*********************************************************************************
 * Globals
 ********************************************************************************/
#define RATIO_SKY             PARALLAX_FLOAT_TO_RATIO(0.2f)
#define RATIO_PLATFORM        PARALLAX_FLOAT_TO_RATIO(1.3f)
#define RATIO_PLATFORM_FAR    PARALLAX_FLOAT_TO_RATIO(0.65f)
#define MAPSCROLL_SPEED       1

u8 g_NewFrame=TRUE;
TMapScrollInfo g_MapInfos[3];


/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void vblFunc(void);


/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();
    
    // set background mode
    ham_SetBgMode(0);
    
    // Will, once activated, trigger the specified interrupt,
    // and then call the function specified
    ham_StartIntHandler(INT_TYPE_VBL, (void *)&vblFunc);
    
    // store background palette
    ham_LoadBGPal(&master_Palette, 256);
    
    // Init tilesets
    ham_bg[0].ti = ham_InitTileSet((void*)&sky_Tiles, SIZEOF_16BIT(sky_Tiles),1,1);
    ham_bg[1].ti = ham_InitTileSet((void*)&platformfar_Tiles, SIZEOF_16BIT(platformfar_Tiles),1,1);
    ham_bg[2].ti = ham_InitTileSet((void*)&platform_Tiles, SIZEOF_16BIT(platform_Tiles),1,1);

    // Init all maps
    g_MapInfos[0] = hel_MapInit(0,  40, 22, 0, (void*)sky_Map);
    g_MapInfos[1] = hel_MapInit(1,  79, 26, 0, (void*)platformfar_Map);
    g_MapInfos[2] = hel_MapInit(2, 128, 32, 0, (void*)platform_Map);
    
    // Set the parallax ratios!
    hel_MapSetParallaxRatio(&g_MapInfos[0], RATIO_SKY, RATIO_SKY);
    hel_MapSetParallaxRatio(&g_MapInfos[1], RATIO_PLATFORM_FAR, RATIO_PLATFORM_FAR);
    hel_MapSetParallaxRatio(&g_MapInfos[2], RATIO_PLATFORM, RATIO_PLATFORM);
    
    // turn on backgrounds
    ham_InitBg(0,1,2,0);
    ham_InitBg(1,1,1,0);
    ham_InitBg(2,1,0,0);
    
    
    // Loop
    while(1)
    {
        if(g_NewFrame)
        {
            TPoint32 ScrollValue={0,0};
            
            // read current joypad state
            hel_PadReadState();

            // Get joypad info and scroll around
            if(hel_Pad.Held.Down)
            {
                ScrollValue.Y=MAPSCROLL_SPEED;
            }

            if(hel_Pad.Held.Up)
            {
                ScrollValue.Y=-MAPSCROLL_SPEED;
            }

            if(hel_Pad.Held.Left)
            {
                ScrollValue.X=-MAPSCROLL_SPEED;
            }

            if(hel_Pad.Held.Right)
            {
                ScrollValue.X=MAPSCROLL_SPEED;
            }


            // Scroll the three map layers
            hel_MapBatchScrollBy(g_MapInfos, ScrollValue.X, ScrollValue.Y, 3);

            g_NewFrame=FALSE;
        }
    }

    return 0;
}

/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc(void)
{
    // Call this (preferably during a VBL interrupt) to commit your
    // ham_obj information to the hardware. Only after you did this your
    // sprite changes will appear on screen.
    ham_CopyObjToOAM();
    
   // It's a new frame yet
    g_NewFrame=1;
}

/* END OF FILE */
